"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TarballService = void 0;

var _postgresLargeObject = require("postgres-large-object");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class TarballService {
  constructor(database, logger, storage, pkg) {
    _defineProperty(this, "database", void 0);

    _defineProperty(this, "logger", void 0);

    _defineProperty(this, "storage", void 0);

    _defineProperty(this, "package", void 0);

    _defineProperty(this, "write", async (name, upload) => {
      this.logger.debug({
        name
      }, '[pg-storage/tarball] write a tarball for package: @{name}');
      const sql = await this.database.sql();
      await sql.begin(async trx => {
        const manager = new _postgresLargeObject.LargeObjectManager(trx);
        const [exists] = await trx`SELECT file FROM tarball WHERE storage = ${this.storage} AND package = ${this.package} AND name = ${name}`;
        const [oid, stream] = await manager.createAndWritableStreamAsync();
        upload.pipe(stream);
        upload.emit('open');
        await new Promise((resolve, reject) => {
          stream.on('error', reject);
          stream.on('finish', resolve);
        });
        await trx`
        INSERT INTO tarball 
          (storage, package, name, file)
        VALUES
          (${this.storage}, ${this.package}, ${name}, ${oid})
        ON CONFLICT (storage, package, name)
        DO UPDATE
        SET 
          file = ${oid},
          updated = NOW()
      `;

        if (exists) {
          await manager.unlinkAsync(exists.file);
        }
      });
      this.logger.debug({
        name
      }, '[pg-storage/tarball] a tarball for package: @{name} written');
    });

    _defineProperty(this, "read", async (name, read) => {
      this.logger.debug({
        name
      }, '[pg-storage/package] read a tarball for package: @{name}');
      const sql = await this.database.sql();
      await sql.begin(async trx => {
        const manager = new _postgresLargeObject.LargeObjectManager(trx);
        const [{
          file
        }] = await trx`
        SELECT file FROM tarball WHERE storage = ${this.storage} AND package = ${this.package} AND name = ${name}
      `;
        const [size, stream] = await manager.openAndReadableStreamAsync(file);
        read.emit('content-length', size);
        read.emit('open');
        stream.pipe(read);
        await new Promise((resolve, reject) => {
          stream.on('end', resolve);
          stream.on('error', reject);
        });
      });
      this.logger.debug({
        name
      }, '[pg-storage/package] a tarball for package: @{name} read');
    });

    _defineProperty(this, "delete", async name => {
      this.logger.debug({
        name
      }, '[pg-storage/package] delete a tarball for name: @{name}');
      const sql = await this.database.sql();
      await sql.begin(async trx => {
        const manager = new _postgresLargeObject.LargeObjectManager(trx);
        const rows = await trx`
        DELETE FROM tarball WHERE storage = ${this.storage} AND ${this.package} AND name = ${name} RETURNING *
      `;

        for (const row of rows) {
          await manager.unlinkAsync(row.file);
        }
      });
      this.logger.debug({
        name
      }, '[pg-storage/package] a tarball for name: @{name} deleted');
    });

    _defineProperty(this, "remove", async () => {
      this.logger.debug({
        package: this.package
      }, '[pg-storage/package] remove all tarball for package: @{package}');
      const sql = await this.database.sql();
      await sql.begin(async trx => {
        const manager = new _postgresLargeObject.LargeObjectManager(trx);
        const rows = await trx`
        DELETE FROM tarball WHERE storage = ${this.storage} AND package = ${this.package} RETURNING *
      `;

        for (const row of rows) {
          await manager.unlinkAsync(row.file);
        }
      });
      this.logger.debug({
        package: this.package
      }, '[pg-storage/package] all tarball for package: @{package} removed');
    });

    this.database = database;
    this.logger = logger;
    this.storage = storage;
    this.package = pkg;
  }

}

exports.TarballService = TarballService;